#include <stdio.h>
#include <unistd.h>
#include "job.h"
#include "master.h"

#define TILE_SIZE 32

typedef struct {
   unsigned int next_tile_row;
   unsigned int next_tile_col;
   unsigned int tile_rows;
} Queue;

bool prepare_next_job(Job* job, Parameters* p, Queue* q) {
   if (q->next_tile_row >= q->tile_rows) return false;
   job->row = q->next_tile_row * TILE_SIZE;
   job->column = q->next_tile_col * TILE_SIZE;
   if (job->column + TILE_SIZE > p->width) {
      job->width = p->width - job->column;
      q->next_tile_row++; q->next_tile_col = 0;
   } else {
      job->width = TILE_SIZE;
      ++q->next_tile_col;
   }
   if (job->row + TILE_SIZE > p->height) {
      job->height = p->height - job->row;
   } else {
      job->height = TILE_SIZE;
   }
   return true;
}

bool master(Parameters* p, int fd) {
   Queue queue = {
      .next_tile_row = 0,
      .next_tile_col = 0,
      .tile_rows = (p->height + (TILE_SIZE - 1)) / TILE_SIZE,
   };
   Job job;
   while (prepare_next_job(&job, p, &queue)) {
      if (write(fd, &job, sizeof(Job)) != sizeof(Job)) {
	 return false;
      }
   }
   return true;
}
